/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * @class Ext.ux.form.ColorPickerField
 * @extends Ext.form.field.Picker This class makes Ext.ux.AdvancedColorPicker available
 *          as a form field.
 * @license: BSD
 * @author: Olav Snoek (extjs id: osnoekie)
 * @constructor Creates a new ColorPickerField
 * @param {Object}
 *            config Configuration options
 * @version 1.0.0
 */

Ext.define( 'Ext.ux.form.ColorPickerField', {
  extend : 'Ext.form.field.Picker',
  //requires : ['Ext.ux.AdvancedColorPicker'], // JMO: commented out to prevent 404 while loading js/extjs/src/ux/AdvancedColorPicker.js
  alias : 'widget.colorfield',
  matchFieldWidth : false,

  invalidValueError : 'Value must be in #RRGGBB format.',


  createPicker : function ()
  {
    var me = this;

    return Ext.create( 'Ext.ux.AdvancedColorPicker', {
      floating : true,
      focusOnShow : true,
      baseCls : Ext.baseCSSPrefix + 'colorpicker',
      listeners : {
        scope : me,
        select : me.onSelect
      }
    } )
  },

  onSelect : function ( picker, value )
  {
    var me = this, hex = '#' + value;

    me.setValue( hex );
    me.fireEvent( 'select', me, hex );
    me.collapse();
  },

  onExpand : function ( picker )
  {
    var me = this, value = me.getValue();
    me.picker.setValue( me.getValue() );
  },


  /**
   * Invoked by the ExtJS validation framework to validate this field.
   * @param processedRawValue the raw field value.
   * @return an array with error messages.
   */
  getErrors : function ( processedRawValue )
  {
    var me = this;
    var errors = me.callParent( arguments );

    var value = me.getValue();

    if ( !Ext.isEmpty( value ) && !me.allowBlank )
    {
      // value must be in the #RRGGBB format
      if ( !/^#[0-9a-f]{6}$/i.test( value ) )
      {
        errors.push( me.invalidValueError );
      }
    }

    return errors;
  }
} );