/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

(function ()
{
  Ext.BLANK_IMAGE_URL = 'images/s.gif';

  Ext.Loader.setConfig( {
    enabled : true, // enables dynamic (on-demand) class-loading
    disableCaching : true, // false = removes the _dc param from all loader's HTTP requests
    paths : {
      'Ext' : 'js/extjs',
      'QDesk' : 'js/qdesk'
    }
  } );

  Ext.require( [
    'QDesk.Config',
    'QDesk.Log',
    'QDesk.Msg',

    'QDesk.enums.ExecNotificationRuleConditionType',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.ExpressionLanguage',
    'QDesk.enums.MessageChannel',
    'QDesk.enums.NotificationSeverity',
    'QDesk.enums.ProxyType',
    'QDesk.enums.WSAuthenticationType',

    'QDesk.util.constant.ActionUrl',

    /* Scheduler actions */
    'QDesk.controller.command.scheduler.StartScheduler',
    'QDesk.controller.command.scheduler.PauseScheduler',
    'QDesk.controller.command.scheduler.StopScheduler',
    'QDesk.controller.command.connection.EditConnectionFolder',
    'QDesk.controller.command.connection.EditSchedulerConnection',
    'QDesk.controller.command.connection.DeleteConnectionFolder',
    'QDesk.controller.command.connection.DeleteSchedulerConnection',

    'QDesk.controller.command.scheduler.quartz.OpenSchedulerView',
    'QDesk.controller.command.settings.EditAppSettings',

    'QDesk.controller.command.scheduler.quartz.RefreshActiveTab',

    /* Jobs tab */
    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',

    /* Triggers tab */
    'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',

    /* Info tab */
    'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoForm',

    /* Exec history tab */
    'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid',

    /* Statistics history tab */
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',

    /* Exec notifications tab */
    'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid',

    'QDesk.store.quartz.JobTriggerGroupStore',

    'QDesk.store.quartz.stats.JobStatisticsTimePeriodStore',
    'QDesk.store.quartz.stats.JobStatisticsTypeStore',

    'QDesk.store.quartz.stats.SchedulerStatisticsTimePeriodStore',
    'QDesk.store.quartz.stats.SchedulerStatisticsTypeStore',

    'QDesk.store.quartz.stats.TriggerStatisticsTimePeriodStore',
    'QDesk.store.quartz.stats.TriggerStatisticsTypeStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.Viewport',

    'Ext.data.Connection',
    'Ext.data.proxy.Ajax',
    'Ext.data.proxy.Server',

    'Ext.form.Basic'
  ] );


  /*
   * Register a global error handler for errors raised throughout the ExtJS framework and this application.
   * Note: to raise an error the application should use the Ext.Error.raise function.
   */
  Ext.Error.handle = function ( err )
  {
    Ext.log( {level : 'error', stack : true}, err );
    return QDesk.util.UxUtils.handleError( err );
  };


  Ext.application( {
    name : 'QDesk',
    appFolder : 'js/qdesk',

    controllers : [
      'navigation.HelpController',
      'navigation.NavigationController',

      'navigation.ConnectionsTreeController',
      'navigation.ConnectionsTreeToolbarController',

      'navigation.SettingsController',

      'scheduler.quartz.SchedulerController',

      'top.SwitchLocaleController',
      'top.LicenseInfoController'
    ],

    models : [
      'AboutData',
      'ConnectionsTreeNode',
      'JSScriptingObjectTreeNode',
      'Locale',
      'MessageMacroTreeNode',
      'NameValuePair',
      'Scheduler',
      'Server',
      'SettingsTreeNode',

      'message.Message',
      'message.MessageAttachment',
      'message.MessageChannelProfile',
      'message.MessageChannelProfilesTreeNode',

      'quartz.ExecHistory',
      'quartz.JobDataMapEntry',
      'quartz.JobDetail',
      'quartz.JobGroup',
      'quartz.Trigger',
      'quartz.TriggerGroup',
      'quartz.TriggerWithJobDetail',
      'quartz.stats.ExecStatData'
    ],

    /* Singleton stores that are automatically instantiated and registered with the Ext.StoreManager */
    stores : [
      'EmptyStore',

      'LocaleStore',
      'ConnectionsTreeStore',

      'enums.ExecNotificationRuleConditionTypeStore',
      'enums.ExecStatusStore',
      'enums.ExpressionLanguageStore',
      'enums.JsLoggingLevelStore',
      'enums.MessageChannelStore',
      'enums.NotificationSeverityStore',
      'enums.ProxyTypeStore',
      'enums.WSAuthenticationTypeStore',

      'quartz.ExecNotificationRuleJSScriptingObjectsStore',
      'quartz.ExecNotificationRuleMessageMacrosStore',
      'quartz.stats.StatisticsTimePeriodLengthStore',
      'quartz.stats.JobStatisticsTypeStore',
      'quartz.stats.SchedulerStatisticsTypeStore',
      'quartz.stats.TriggerStatisticsTypeStore'
    ],

    /*
     * Executed by ExtJS from Ext.onReady
     */
    launch : function ()
    {
      /* Start of ExtJS overrides */

      /*
       * Fix for the wrong tooltip width in ExtJS 4.2 in Chrome
       * http://www.sencha.com/forum/showthread.php?257201-4.2-RC-Tooltip-cuts-off-content/page3
       */
      Ext.tip.Tip.prototype.minWidth = 'auto';
      //delete Ext.tip.Tip.prototype.minWidth;

      Ext.override( Ext.data.proxy.Server, {
        /* Parameter names used in Ajax data store requests */
        startParam : 'startAt',
        groupParam : 'groupBy',
        sortParam : 'sort',
        filterParam : 'filter',

        noCache : true  // true is default (a _dc param s appended to all proxy request URLs)
      } );

      Ext.override( Ext.data.proxy.Ajax, {
        /* Default timeout in ms for all Ajax proxies */
        timeout : 1000 * QDesk.Config.get( 'ajaxTimeout' )
      } );

      Ext.override( Ext.data.Connection, {
        disableCaching : false, // false is default
        /* Default connect timeout in ms */
        timeout : 1000 * QDesk.Config.get( 'ajaxTimeout' )
      } );

      Ext.override( Ext.form.Basic, {
        /* Default timeout (in seconds) for all forms (used by the form load operation) */
        timeout : QDesk.Config.get( 'ajaxTimeout' )
      } );


      Ext.override( Ext.draw.engine.ImageExporter, {
        /*
         * Replaces the default ExtJS SVG export action URL "http://svg.sencha.io".
         */
        defaultUrl : QDesk.util.constant.ActionUrl.svgActionRasterize
      } );


      /*
       * Forms are validated after rendering so that invalid fields are immediately visually
       * distinguished.
       *
       * Form validation also occurs upon form field change.
       */
      Ext.override( Ext.form.Panel, {
        initComponent : function ()
        {
          var me = this;

          me.callParent( arguments );

          me.on( 'afterrender', function ()
          {
            me.isValid();
          } );

          Ext.each( Ext.ComponentQuery.query( 'field', me ), function ( field )
          {
            field.on( 'change', function ()
            {
              me.isValid();
            } );
          } );
        }
      } );


      /**
       * setCursorPosition and getCursorPosition for textfield, textarea etc.
       * http://www.sencha.com/forum/showthread.php?95486-Cursor-Position-in-TextField
       */
      Ext.override( Ext.form.field.Text, {

        setCaretPosition : function ( pos )
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            el.focus();
            el.setSelectionRange( pos, pos );
          }
          else if ( el.createTextRange )
          {
            var range = el.createTextRange();
            range.move( "character", pos );
            range.select();
          }
          else
          {
            //throw 'setCaretPosition() not supported';
          }
        },

        getCaretPosition : function ()
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            return el.selectionStart;
          }
          else if ( document.selection && el.createTextRange )
          {
            var range = document.selection.createRange();
            range.collapse( true );
            range.moveStart( "character", -el.value.length );
            return range.text.length;
          }
          else
          {
            //throw 'getCaretPosition() not supported';
          }
        }
      } );


      /**
       * Localization of Ext.ux.form.ColorPickerField texts.
       */
      Ext.override( Ext.ux.form.ColorPickerField, {
        invalidValueError : QDesk.Msg.get( 'Ext.ux.form.ColorPickerField.invalidValueError' )
      } );


      /**
       * Localization of Ext.ux.TreeIconCombo texts.
       */
      Ext.override( Ext.ux.form.TreeIconCombo, {
        emptyValueText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        valueNotFoundText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        loadFailedText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadFailedText' ),
        loadingText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadingText' )
      } );


      /* Temporary fixes */

      /*
       * Fixes a bug in ExtJS 4.2 Beta with occasional closing of tabs (lastVisibleColumn not defined).
       */
      Ext.override( Ext.grid.header.Container, {
        getGridColumns : function ( refreshCache /* private - used in recursion*/, inResult, hiddenAncestor )
        {
          if ( !refreshCache && this.gridDataColumns )
          {
            return this.gridDataColumns;
          }

          var me = this,
              result = inResult || [],
              items, i, len, item,
              lastVisibleColumn;

          hiddenAncestor = hiddenAncestor || me.hidden;
          if ( me.items )
          {
            items = me.items.items;
            for ( i = 0, len = items.length; i < len; i++ )
            {
              item = items[i];
              if ( item.isGroupHeader )
              {
                item.getGridColumns( true, result, hiddenAncestor );
              }
              else
              {
                item.hiddenAncestor = hiddenAncestor;
                result.push( item );
              }
            }
          }
          me.gridDataColumns = result;

          // If top level, correct first and last visible column flags
          if ( !inResult && len )
          {
            // Set firstVisible and lastVisible flags
            for ( i = 0, len = result.length; i < len; i++ )
            {
              item = result[i];
              item.isFirstVisible = item.isLastVisible = false;
              if ( !item.hidden || item.hiddenAncestor )
              {
                if ( !lastVisibleColumn )
                {
                  item.isFirstVisible = true;
                }
                lastVisibleColumn = item;
              }
            }

            // JMO: added lastVisibleColumn null check
            if ( lastVisibleColumn != null )
            {
              lastVisibleColumn.isLastVisible = true;
            }
          }

          return result;
        }
      } );
      /* End of ExtJS 4.2 tab close error */

      /*
       * Bug in ExtJs 4.2.0.663 that pops up when the font selection combo is disabled.
       *
       * See bug report at http://www.sencha.com/forum/showthread.php?258836
       */
//      Ext.override( Ext.form.field.HtmlEditor, {
//
//        initDefaultFont : function ()
//        {
//          // It's not ideal to do this here since it's a write phase, but we need to know
//          // what the font used in the textarea is so that we can setup the appropriate font
//          // options in the select box. The select box will reflow once we populate it, so we want
//          // to do so before we layout the first time.
//
//          var me = this,
//              selIdx = 0,
//              fonts, font, select,
//              option, i, len, lower;
//
//          /* JMO: added me.down( '#fontSelect' ) != null check */
//          if ( !me.defaultFont )
//          {
//            font = me.textareaEl.getStyle( 'font-family' );
//            font = Ext.String.capitalize( font.split( ',' )[0] );
//            fonts = Ext.Array.clone( me.fontFamilies );
//            Ext.Array.include( fonts, font );
//            fonts.sort();
//            me.defaultFont = font;
//
//            if ( me.down( '#fontSelect' ) )
//            {
//              select = me.down( '#fontSelect' ).selectEl.dom;
//              for ( i = 0, len = fonts.length; i < len; ++i )
//              {
//                font = fonts[i];
//                lower = font.toLowerCase();
//                option = new Option( font, lower );
//                if ( font == me.defaultFont )
//                {
//                  selIdx = i;
//                }
//                option.style.fontFamily = lower;
//                select.add( option );
//              }
//              // Old IE versions have a problem if we set the selected property
//              // in the loop, so set it after.
//              select.options[selIdx].selected = true;
//            }
//          }
//        }
//      } );

      /* End of ExtJS overrides */

      Ext.create( 'QDesk.view.Viewport' );
    }
  } );
})();
