/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Adds a new connection to the specified parent folder in the
 * connections tree panel.
 */
Ext.define( 'QDesk.controller.command.connection.AddSchedulerConnection', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.navigation.connections.EditSchedulerConnectionFormMode',
    'QDesk.view.navigation.connections.EditSchedulerConnectionWindow'
  ],

  /*
   * Required ID of the parent folder the new connection should be added to.
   */
  parentFolderId : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.enums.scheduler.SchedulerType.
   */
  schedulerType : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.parentFolderId )
      Ext.Error.raise( 'Required config parameter missing: parentFolderId' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditSchedulerConnectionWindow', {
      qdesk : {
        mode : QDesk.view.navigation.connections.EditSchedulerConnectionFormMode.ADD,
        schedulerType : me.schedulerType
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    // Discover button
    formPanel.down( '#discoverButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.DiscoverSchedulers', {
        window : window,
        schedulerType : me.schedulerType
      } ).execute();
    }, me );

    // Test Connection button
    formPanel.down( '#testButton' ).on( 'click', function ()
    {
      var me = this;
      Ext.create( 'QDesk.controller.command.connection.TestSchedulerConnection', {
        window : window,
        schedulerType : me.schedulerType
      } ).execute();
    }, me );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );

    window.show();
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var form = window.down( '#form' ).getForm();

    var fieldValues = form.getFieldValues();

    var connection = Ext.create( 'QDesk.model.Connection',
        {
          name : fieldValues.name,
          server : {
            host : fieldValues.host,
            port : fieldValues.port,
            authentication : {
              username : fieldValues.username,
              password : fieldValues.password
            }
          },
          schedulerType : me.schedulerType,
          schedulerObjectName : fieldValues.schedulerObjectName
        }
    );

    if ( fieldValues.username == '' && fieldValues.password == '' )
      connection.get( 'server' ).authentication = null;

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.schedulerConnectionActionAddConnection,
      params : {
        parentFolderId : me.parentFolderId,
        connection : Ext.encode( connection.data )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.connection.AddSchedulerConnection.waitMsg', fieldValues.name ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.connection.AddSchedulerConnection.successMsg',
                  fieldValues.name ),
              function ()
              {
                me.application.fireEvent( 'connectionAdded', {
                  connectionId : json.data.id
                } );
              } );

          me._refreshTargetFolder();
        }
        else
        {
          QDesk.util.UxUtils.showServerError( json );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action );
      }
    } );
  },


  /**
   * Reloads the connection's parent folder.
   */
  _refreshTargetFolder : function ()
  {
    var me = this;

    // reload the connection's parent folder in the connections tree view
    Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
      folderId : me.parentFolderId
    } ).execute();
  }
} );
