/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


/**
 * Scheduler connection related utility methods.
 */
Ext.define( 'QDesk.controller.command.connection.ConnectionUtils', {
  singleton : true,


  /**
   * Returns the item ID of the tab panel of the specified connection.
   * @param connectionId a connection ID.
   */
  getSchedulerTabItemId : function ( connectionId )
  {
    return 'schedulerTabPanel-' + connectionId;
  },


  /**
   * Returns the tab panel for the specified Quartz connection ID.
   * @param connectionId a connection ID.
   */
  getSchedulerTab : function ( connectionId )
  {
    var tabPanelItemId = QDesk.controller.command.connection.ConnectionUtils.getSchedulerTabItemId( connectionId );
    var tabPanels = Ext.ComponentQuery.query( '#mainTabPanel > #' + tabPanelItemId );
    // assert tabPanels.size <= 1
    return Ext.isEmpty( tabPanels ) ? null : tabPanels[0];
  },


  /**
   * Returns the node ID of the specified connection.
   * @param connectionId a connection ID.
   * @return the node ID of the specified connection.
   */
  getConnectionNodeId : function ( connectionId )
  {
    return 'connection-' + connectionId;
  },


  /**
   * Returns the name of the connection with the specified ID.
   * @param connectionId a connection ID.
   */
  getConnectionName : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified connection, or null if not found.
   * @param connectionId a connection ID.
   * @return the parent folder ID, or null.
   */
  getConnectionParentFolderId : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode ? connectionNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the node ID of the specified folder.
   * @param folderId a folder ID.
   * @return the node ID of the specified folder.
   */
  getFolderNodeId : function ( folderId )
  {
    return folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID ?
        QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID : 'folder-' + folderId;
  },


  /**
   * Returns the name of the folder with the specified ID.
   * @param folderId a folder ID.
   */
  getFolderName : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode.get( 'name' );
  },


  /**
   * Returns the ID of the parent folder of the specified folder, or null if not found.
   * @param folderId a folder ID.
   * @return the parent folder ID, or null.
   */
  getFolderParentFolderId : function ( folderId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getFolderNodeId( folderId );
    var folderNode = store.getNodeById( nodeId );
    return folderNode ? folderNode.parentNode.get( 'folderId' ) : null;
  },


  /**
   * Returns the scheduler type for the specified connection ID.
   * @param connectionId a connection ID.
   * @return the scheduler type (QDesk.enums.scheduler.SchedulerType value).
   */
  getSchedulerType : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var nodeId = QDesk.controller.command.connection.ConnectionUtils.getConnectionNodeId( connectionId );
    var connectionNode = store.getNodeById( nodeId );
    return connectionNode.get( 'schedulerType' );
  },


  /**
   * Returns true if the QuartzDesk JVM Agent was available when the scheduler
   * tab was opened, false otherwise.
   * @param connectionId a connection ID.
   * @return true if the QuartzDesk JVM Agent was available when the scheduler
   * tab was opened, false otherwise.
   */
  isJvmAgentAvailable : function ( connectionId )
  {
    var me = this;

    var schedulerTab = me.getSchedulerTab( connectionId );
    return schedulerTab.isJvmAgentAvailable();
  }
} );
