/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Scheduler discovery used by the AddConnection and EditConnection commands.
 */
Ext.define( 'QDesk.controller.command.connection.DiscoverSchedulers', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.ServerErrorType',
    'QDesk.util.UxUtils'
  ],

  /*
   * Required config parameter with the AddConnection, or EditConnection window
   * that initiated the discovery.
   */
  window : undefined,

  /**
   * Required config parameter with the scheduler type.
   * See QDesk.enums.scheduler.SchedulerType.
   */
  schedulerType : undefined,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.window )
      Ext.Error.raise( 'Required config parameter missing: window' );

    if ( !config.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: schedulerType' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var form = me.window.down( '#form' );

    var schedulerObjectNameCombo = form.down( '#schedulerObjectName' );
    schedulerObjectNameCombo.setValue( '' );

    var fieldValues = form.getForm().getFieldValues();

    var server = Ext.create( 'QDesk.model.Server',
        {
          host : fieldValues.host,
          port : fieldValues.port,
          authentication : {
            username : fieldValues.username,
            password : fieldValues.password
          }
        } );

    QDesk.util.UxUtils.maskEl( me.window.getEl(),
        QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.maskMsg' ) );

    Ext.Ajax.request( {
      url : QDesk.util.constant.ActionUrl.commonSchedulerActionDiscover,
      params : {
        schedulerType : me.schedulerType,
        server : Ext.encode( server.data )
      },
      callback : function ( opts, success, resp )
      {
        QDesk.util.UxUtils.unmaskEl( me.window.getEl() );

        if ( success )
        {
          var json = Ext.decode( resp.responseText );
          if ( json.success )
          {
            schedulerObjectNameCombo.store.loadData( json.schedulers );

            if ( json.schedulers.length == 0 )
            {
              QDesk.util.UxUtils.showInfoDialog(
                  QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.noSchedulersFoundMsg' ) );
            }
            else
            {
              schedulerObjectNameCombo.setValue( json.schedulers[0].objectName );

              QDesk.util.UxUtils.showInfoDialog(
                  QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.schedulersFoundMsg',
                      json.schedulers.length ) );
            }
          }
          else
          {
            schedulerObjectNameCombo.store.loadData( [] );
            // application errors are shown in an info dialogs
            if ( json.errorType && json.errorType == QDesk.enums.ServerErrorType.APPLICATION )
            {
              QDesk.util.UxUtils.showInfoDialog( json.errors[0].msg );
            }
            else
            {
              QDesk.util.UxUtils.showServerError( json );
            }
          }
        }
        else
        {
          schedulerObjectNameCombo.store.loadData( [] );
          QDesk.util.UxUtils.showErrorDialog(
              QDesk.Msg.get( 'QDesk.controller.command.connection.DiscoverSchedulers.errorMsg' ) );
        }
      }
    } )
  }
} );
