/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Opens a trigger job window for the specified job and triggers it.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.util.UxUtils',
    'QDesk.util.quartz.QuartzUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode',
    'QDesk.view.main.scheduler.quartz.jobs.JobDetailWindow'
  ],

  /*
   * Required connection ID of the scheduler whose tab should be refreshed.
   */
  connectionId : undefined,

  /**
   * Required config parameter with the job detail of the job to trigger.
   */
  jobDetail : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.jobDetail )
      Ext.Error.raise( 'Required config parameter missing: jobDetail' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var jobDetailClone = QDesk.util.quartz.QuartzUtils.cloneJobDetail( me.jobDetail );

    var formWindow = Ext.widget( 'QuartzJobDetailWindow', {
      title : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob.windowTitle' ),

      qdesk : {
        jobDetailFormMode : QDesk.view.main.scheduler.quartz.jobs.JobDetailFormMode.TRIGGER,
        jobDetail : jobDetailClone,
        callback : function ( jobDetail )
        {
          var jobGroup = jobDetail.get( 'group' );
          var jobName = jobDetail.get( 'name' );

          var form = formWindow.down( '#jobDetailForm' ).getForm();
          form.doAction( 'ParamsSubmit', {
            url : QDesk.util.constant.ActionUrl.quartz.jobActionTrigger,
            params : {
              connectionId : me.connectionId,
              jobDetail : Ext.encode( jobDetail.data )
            },
            waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob.waitMsg',
                jobGroup.name, jobName ),
            success : function ( form, action )
            {
              formWindow.close();

              QDesk.util.UxUtils.showInfo( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.jobs.TriggerJob.successMsg',
                  jobGroup.name, jobName ),
                  function ()
                  {
                    me.application.fireEvent( 'quartzJobTriggered', {
                      connectionId : me.connectionId,
                      jobDetail : me.jobDetail
                    } );

                    // invoke the success callback handler
                    if ( me.success && Ext.isFunction( me.success ) )
                    {
                      me.success.apply( me );
                    }
                  } );
            },
            failure : function ( form, action )
            {
              QDesk.util.UxUtils.showExtJsActionError( action,
                  function ()
                  {
                    // invoke the failure callback handler
                    if ( me.failure && Ext.isFunction( me.failure ) )
                    {
                      me.failure.apply( me );
                    }
                  } );
            }
          } );
        },
        scope : me
      }
    } );

    formWindow.show();
  }
} );
