/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Edits an existing notification rule with the specified ID.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.ExecNotificationRuleUtils',
    'QDesk.enums.scheduler.SchedulerType',
    'QDesk.util.FormatUtils',
    'QDesk.util.ModelUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleFormMode',
    'QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleWindow'
  ],

  /*
   * Required ID of the scheduler connection.
   */
  connectionId : undefined,

  /**
   * Required ID of the notification rule to edit.
   */
  notificationRuleId : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( !config.notificationRuleId )
      Ext.Error.raise( 'Required config parameter missing: notificationRuleId' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var window = Ext.widget( 'EditExecNotificationRuleWindow', {
      qdesk : {
        mode : QDesk.view.main.scheduler.common.notif.exec.EditExecNotificationRuleFormMode.EDIT,
        schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
      }
    } );

    var formPanel = window.down( '#form' );

    window.on( 'afterrender', function ()
    {
      var nameField = formPanel.down( '#name' );
      nameField.focus( false, QDesk.util.constant.Const.FOCUS_DELAY );
    } );

    // Save/Update button
    formPanel.down( '#actionButton' ).on( 'click', function ()
    {
      me._submitForm( window );
    } );


    /* Load the form data */
    QDesk.util.UxUtils.maskBody( QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule.maskLoadMsg' ) );

    formPanel.load( {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionGet,
      scope : me,
      params : {
        connectionId : me.connectionId,
        notificationRuleId : me.notificationRuleId
      },
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          // set enabledOnWeekdays checkboxes
          Ext.each( Ext.ComponentQuery.query( '#enabledOnWeekdaysFieldSet checkbox', formPanel ),
              function ( checkbox )
              {
                checkbox.setValue( Ext.Array.contains( json.data.enabledOnWeekdays, checkbox.qdesk.value ) );
              } );

          // set enabledFromTime
          if ( json.data.enabledFromTime )
          {
            var enabledFromDate = QDesk.util.FormatUtils.parseTime( json.data.enabledFromTime );
            formPanel.down( '#enabledFromTime' ).setValue( enabledFromDate );
          }

          // set enabledToTime fields
          if ( json.data.enabledToTime )
          {
            var enabledToDate = QDesk.util.FormatUtils.parseTime( json.data.enabledToTime );
            formPanel.down( '#enabledToTime' ).setValue( enabledToDate );
          }

          QDesk.controller.ExecNotificationRuleUtils.populateFormMessageChannelConfig( formPanel,
              json.data.messageChannel,
              json.data.messageChannelConfig );

          QDesk.util.UxUtils.unmaskBody();
          window.show();
        }
        else
        {
          QDesk.util.UxUtils.unmaskBody();
          window.close();
          QDesk.util.UxUtils.showServerError( json );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.unmaskBody();
        window.close();
        QDesk.util.UxUtils.showExtJsActionError( action );
      }
    } );
  },


  /**
   * Submits the form.
   * @param window the edit connection window.
   */
  _submitForm : function ( window )
  {
    var me = this;

    var formPanel = window.down( '#form' );

    var form = formPanel.getForm();
    var fieldValues = form.getFieldValues();

    var notificationRule = QDesk.controller.ExecNotificationRuleUtils.createNotificationRule( fieldValues );

    form.doAction( 'ParamsSubmit', {
      url : QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionUpdate,
      params : {
        connectionId : me.connectionId,
        notificationRule : QDesk.util.ModelUtils.encode( notificationRule )
      },
      waitMsg : QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule.waitMsg',
          fieldValues.name ),
      success : function ( form, action )
      {
        var json = Ext.decode( action.response.responseText );
        if ( json.success )
        {
          window.close();

          QDesk.util.UxUtils.showInfo(
              QDesk.Msg.get( 'QDesk.controller.command.scheduler.quartz.notif.exec.EditExecNotificationRule.successMsg',
                  fieldValues.name ),
              function ()
              {
                me.application.fireEvent( 'quartzNotificationRuleUpdated', {
                  connectionId : me.connectionId
                } );

                // invoke the success callback handler
                if ( me.success && Ext.isFunction( me.success ) )
                {
                  me.success.apply( me );
                }
              } );
        }
        else
        {
          QDesk.util.UxUtils.showServerError( json,
              function ()
              {
                // invoke the failure callback handler
                if ( me.failure && Ext.isFunction( me.failure ) )
                {
                  me.failure.apply( me );
                }
              } );
        }
      },
      failure : function ( form, action )
      {
        QDesk.util.UxUtils.showExtJsActionError( action,
            function ()
            {
              // invoke the failure callback handler
              if ( me.failure && Ext.isFunction( me.failure ) )
              {
                me.failure.apply( me );
              }
            } );
      }
    } );
  }
} );
