/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.controller.navigation.ConnectionsTreeController', {
  extend : 'Ext.app.Controller',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.command.connection.MoveConnectionFolder',
    'QDesk.controller.command.connection.MoveSchedulerConnection',
    'QDesk.enums.scheduler.SchedulerType',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.Const',
    'QDesk.util.constant.ActionUrl'
  ],

  init : function ()
  {
    var me = this;

    // key is a component query, value is a listener configuration
    me.control( {
      // tree node click
      '#connectionsTreePanel' : {
        itemcontextmenu : function ( view, record, item, index, event )
        {
          var nodeId = record.get( 'id' );
          event.stopEvent();

          var nodeType = record.get( 'nodeType' );
          if ( nodeId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID || nodeType == 'folder' )
          {
            var folderCtxMenu = me._createFolderCtxMenu( record );
            var x = event.browserEvent.clientX;
            var y = event.browserEvent.clientY;
            folderCtxMenu.showAt( [x, y] );
          }
          else if ( nodeType == 'connection' )
          {
            var connectionCtxMenu = me._createConnectionCtxMenu( record );
            var x = event.browserEvent.clientX;
            var y = event.browserEvent.clientY;
            connectionCtxMenu.showAt( [x, y] );
          }
        },

        itemdblclick : function ( view, record )
        {
          var nodeType = record.get( 'nodeType' );
          if ( nodeType == 'folder' )
          {
            me.application.fireEvent( 'connectionsTreeFolderDblClicked', record );
          }
          else if ( nodeType == 'connection' )
          {
            me._openSchedulerView( record );
          }
        }
      }
    } );

    me.application.addListener( 'schedulerStarted', me.onSchedulerStarted, me );
    me.application.addListener( 'schedulerPaused', me.onSchedulerPaused, me );
    me.application.addListener( 'schedulerStopped', me.onSchedulerStopped, me );

    me.callParent( arguments );
  },


  onLaunch : function ( application )
  {
    var me = this;

    me.callParent( arguments );

    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];

    var treeView = treePanel.getView();

    treeView.on( 'beforedrop', function ( node, data, overModel, dropPosition, dropFunction, eOpts )
    {
      var draggedRecord = data.records[0];
      var targetRecord = overModel;

      var draggedRecordParent = draggedRecord.parentNode;

      // if node is dragged to its existing parent, then we cancel the operation
      if ( draggedRecordParent.get( 'id' ) == targetRecord.get( 'id' ) )
        return false;

      if ( !me._checkNodeMove( draggedRecord, targetRecord ) )
      {
        return false;
      }

      me._onNodeDrag( draggedRecord, targetRecord );

      return 1;
    } );

    var treeStore = treePanel.getStore();

    /*
     * Swap the initial tree store url Empty.action for the real action url
     * and fire an event to request schedulers tree reloading. The delayed task
     * is used to workaround a problem in IE and Chrome where the tree panel's
     * "Loading..." message does not pop up during the initial tree store load.
     */
    treeStore.getProxy().url = QDesk.util.constant.ActionUrl.connectionActionTree;

    // load the connections tree
    var reloadTask = new Ext.util.DelayedTask( function ()
    {
      Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
      } ).execute();
    } );
    reloadTask.delay( 200 );
  },


  /**
   * Invoked when a scheduler has been started.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  onSchedulerStarted : function ( eventData )
  {
    /*
     var me = this;

     Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
     folderId : me._getParentFolderForConnection( eventData.connectionId )
     } ).execute();
     */
  },


  /**
   * Invoked when a scheduler has been paused.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  onSchedulerPaused : function ( eventData )
  {
    /*
     var me = this;

     Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
     folderId : me._getParentFolderForConnection( eventData.connectionId )
     } ).execute();
     */
  },


  /**
   * Invoked when a scheduler has been stopped.
   * @param eventData an event data object with the following attributes:
   * <pre>
   *   connectionId         : connection ID
   * </pre>
   */
  onSchedulerStopped : function ( eventData )
  {
    /*
     var me = this;

     Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
     folderId : me._getParentFolderForConnection( connectionId )
     } ).execute();
     */
  },


  /**
   * Returns the ID of the parent folder of the specified connection.
   * @param connectionId a connection ID.
   * @private the parent folder ID, or null if the connection was not found.
   */
  _getParentFolderForConnection : function ( connectionId )
  {
    var treePanel = Ext.ComponentQuery.query( '#connectionsTreePanel' )[0];
    var store = treePanel.getStore();

    var connectionNode = store.getNodeById( connectionId );
    var parentFolderNode = connectionNode ? connectionNode.parentNode : null;
    return parentFolderNode ? parentFolderNode.data.id : null;
  },


  /**
   * Creates and returns the context menu for a folder node in the connections tree.
   * @param folderRecord a folder record (see QDesk.model.ConnectionsTreeNode}.
   * @return the created context menu.
   */
  _createFolderCtxMenu : function ( folderRecord )
  {
    var me = this;

    var folderId = folderRecord.get( 'folderId' );
    var isRootFolder = folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_FOLDER_ID;

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.folderCtxMenuAddFolderOrConnection' ),
          menu : {
            items : [
              {
                text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.folderCtxMenuAddFolder' ),
                iconCls : 'addFolder',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.connection.AddConnectionFolder', {
                    parentFolderId : folderId
                  } ).execute();
                }
              },
              {
                text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.folderCtxMenuAddConnection' ),
                iconCls : 'addConnection',
                handler : function ()
                {
                  Ext.create( 'QDesk.controller.command.connection.AddSchedulerConnection', {
                    parentFolderId : folderId,
                    schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
                  } ).execute();
                }
              }
            ]
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.folderCtxMenuRename' ),
          iconCls : 'renameFolder',
          disabled : isRootFolder,
          handler : function ()
          {
            if ( folderId == QDesk.util.constant.Const.CONNECTION_TREE_ROOT_ID )
            {
              // ignore, root node cannot be renamed
            }
            else
            {
              Ext.create( 'QDesk.controller.command.connection.EditConnectionFolder', {
                folderId : folderId,
                folderName : folderRecord.get( 'name' )
              } ).execute();
            }
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.folderCtxMenuDelete' ),
          iconCls : 'deleteFolder',
          disabled : isRootFolder,
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.DeleteConnectionFolder', {
              folderId : folderId
            } ).execute();
          }
        },
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.folderCtxMenuRefresh' ),
          iconCls : 'refreshFolder',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.RefreshConnectionFolder', {
              folderId : folderId
            } ).execute();
          }
        }
      ]
    } );
  },


  /**
   * Creates and returns the context menu for a connection node in the connections tree.
   * @param connectionRecord a folder record (see QDesk.model.ConnectionsTreeNode}.
   * @return the created context menu.
   */
  _createConnectionCtxMenu : function ( connectionRecord )
  {
    var me = this;

    var connectionId = connectionRecord.get( 'connectionId' );
    var schedulerStatus = connectionRecord.get( 'schedulerStatus' );

    return Ext.create( 'Ext.menu.Menu', {
      items : [
        // open connection
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.connectionCtxMenuOpen' ),
          iconCls : 'openConnection',
          disabled : schedulerStatus == QDesk.enums.scheduler.SchedulerStatus.UNREACHABLE,
          handler : function ()
          {
            me._openSchedulerView( connectionRecord )
          }
        },
        // edit connection
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.connectionCtxMenuEdit' ),
          iconCls : 'editConnection',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.EditSchedulerConnection', {
              connectionId : connectionId,
              schedulerType : QDesk.enums.scheduler.SchedulerType.QUARTZ
            } ).execute();
          }
        },
        // delete connection
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.connectionCtxMenuDelete' ),
          iconCls : 'deleteConnection',
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.connection.DeleteSchedulerConnection', {
              connectionId : connectionId
            } ).execute();
          }
        },
        '-',
        // start scheduler
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.connectionCtxMenuStart' ),
          iconCls : 'startScheduler',
          disabled : schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.PAUSED,
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.StartScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        },
        // pause scheduler
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.connectionCtxMenuPause' ),
          iconCls : 'pauseScheduler',
          disabled : schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.STARTED,
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.PauseScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        },
        // stop scheduler
        {
          text : QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.connectionCtxMenuStop' ),
          iconCls : 'stopScheduler',
          disabled : (schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.STARTED &&
                  schedulerStatus != QDesk.enums.scheduler.SchedulerStatus.PAUSED),
          handler : function ()
          {
            Ext.create( 'QDesk.controller.command.scheduler.StopScheduler', {
              connectionId : connectionId
            } ).execute();
          }
        }
      ]
    } );
  },


  _openSchedulerView : function ( record )
  {
    var me = this;

    var schedulerStatus = record.get( 'schedulerStatus' );

    if ( (schedulerStatus == QDesk.enums.scheduler.SchedulerStatus.UNREACHABLE ) )
    {
      // connection unreachable
      QDesk.util.UxUtils.showInfoDialog( QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.schedulerUnreachableMsg' ) );
    }
    else
    {
      var connectionId = record.get( 'connectionId' );
      var schedulerType = record.get( 'schedulerType' );

      if ( schedulerType == QDesk.enums.scheduler.SchedulerType.QUARTZ )
      {
        Ext.create( 'QDesk.controller.command.scheduler.quartz.OpenSchedulerView', {
          connectionId : connectionId
        } ).execute();
      }
      else
      {
        Ext.Error.raise( 'Unsupported scheduler type: ' + schedulerType );
      }
    }
  },


  /**
   * Checks if the specified record can be dragged to the specified target record.
   * @param draggedRecord the dragged folder/connection record.
   * @param targetRecord the target record (target folder).
   * @return {boolean} true if the dragged folder/connection can be moved, false otherwise.
   * @private
   */
  _checkNodeMove : function ( draggedRecord, targetRecord )
  {
    var canBeMoved = true;

    var targetFolderId = targetRecord.get( 'folderId' );
    var targetFolderName = targetRecord.get( 'name' );

    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      var folderName = draggedRecord.get( 'name' );

      Ext.each( targetRecord.childNodes, function ( node )
      {
        if ( node.get( 'nodeType' ) == 'folder' && node.get( 'name' ) == folderName )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.folderCannotBeMovedMsg',
              folderName, targetFolderName ) );

          canBeMoved = false;
          return false;
        }
      } );
    }
    else if ( nodeType == 'connection' )
    {
      var connectionId = draggedRecord.get( 'connectionId' );
      var connectionName = draggedRecord.get( 'name' );

      Ext.each( targetRecord.childNodes, function ( node )
      {
        if ( node.get( 'nodeType' ) == 'connection' && node.get( 'name' ) == connectionName )
        {
          QDesk.util.UxUtils.showWarningDialog( QDesk.Msg.get( 'QDesk.controller.navigation.ConnectionsTreeController.connectionCannotBeMovedMsg',
              connectionName, targetFolderName ) );

          canBeMoved = false;
          return false;
        }
      } );
    }

    return canBeMoved;
  },


  /**
   * Invoked when a folder, or connection in the connections tree panel has been dragged.
   * @param draggedRecord the dragged folder/connection record.
   * @param targetRecord the target record (target folder).
   * @private
   */
  _onNodeDrag : function ( draggedRecord, targetRecord )
  {
    var targetFolderId = targetRecord.get( 'folderId' );
    var nodeType = draggedRecord.get( 'nodeType' );
    if ( nodeType == 'folder' )
    {
      var folderId = draggedRecord.get( 'folderId' );
      Ext.create( 'QDesk.controller.command.connection.MoveConnectionFolder', {
        folderId : folderId,
        targetFolderId : targetFolderId
      } ).execute();
    }
    else if ( nodeType == 'connection' )
    {
      var connectionId = draggedRecord.get( 'connectionId' );
      Ext.create( 'QDesk.controller.command.connection.MoveSchedulerConnection', {
        connectionId : connectionId,
        targetFolderId : targetFolderId
      } ).execute();
    }
  }
} );
