/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Various utility methods for ExtJS models.
 */
Ext.define( 'QDesk.util.ModelUtils', {
  requires : [
    'QDesk.Log'
  ],

  singleton : true,

  /**
   * Encodes the data wrapped in the specified ExtJS model instance.
   * This method is useful when POSTing extJS model instances as
   * serialized JSON objects.
   * @param model an ExtJS model instance.
   * @return the JSON serialized model instance.
   */
  encode : function ( model )
  {
    var me = this;
    var clone = me._cloneAndDropEmpty( model.data );
    return Ext.encode( clone );
  },


  /**
   * Deep clones the specified object and drops
   * all properties with an empty value.
   * @param o an object to clone.
   * @return the clone result.
   */
  _cloneAndDropEmpty : function ( o )
  {
    var me = this;

    if ( !o || 'object' !== typeof o )
    {
      return o;
    }
    if ( 'function' === typeof o.clone )
    {
      return o.clone();
    }
    var c = '[object Array]' === Object.prototype.toString.call( o ) ? [] : {};
    var p, v;
    for ( p in o )
    {
      if ( o.hasOwnProperty( p ) )
      {
        v = o[p];
        if ( v && Ext.typeOf(v) == 'date' )
        {
          c[p] = new Date( v.getTime());
        }
        else if ( v && 'object' === typeof v )
        {
          c[p] = me._cloneAndDropEmpty( v );
        }
        else
        {
          // copy only non-empty properties
          if ( !Ext.isEmpty( v ) )
            c[p] = v;
        }
      }
    }
    return c;
  }
} );
