/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Base class for all scheduler tabs.
 */
Ext.define( 'QDesk.view.base.scheduler.BaseSchedulerTab', {
  extend : 'Ext.tab.Panel',

  closable : 'true',

  qdesk : {
    /*
     * Required config parameter with the ID of the scheduler connection.
     */
    connectionId : undefined,

    /*
     * Required config parameter with a flag indicating whether the QuartzDesk JVM agent
     * was available when opening the scheduler tab.
     */
    jvmAgentAvailable : undefined
  },


  /**
   * Returns the connection ID this tab has been created for.
   * @return the connection ID.
   */
  getConnectionId : function ()
  {
    return this.qdesk.connectionId;
  },


  /**
   * Returns true if the QuartzDesk JVM Agent was available when opening the scheduler tab, false otherwise.
   * @return true if the QuartzDesk JVM Agent was available when opening the scheduler tab, false otherwise.
   */
  isJvmAgentAvailable : function ()
  {
    return this.qdesk.jvmAgentAvailable;
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( config.qdesk.jvmAgentAvailable == null )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentAvailable' );

    me.callParent( [config] );
  }
} );
