/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.enums.MessageChannel',
    'QDesk.store.message.MessageChannelProfilesTreeStore',
    'QDesk.util.EnumUtils',
    'QDesk.util.constant.Const'
  ],

  alias : 'widget.MessageChannelProfilesTreePanel',

  cls : 'messageChannelProfilesTreePanel',

  //autoScroll : true,

  viewConfig : {
    copy : false,

    plugins : {
      ptype : 'treeviewdragdrop',
      appendOnly : true,
      dragText : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.dragText' )
    }
  },

  displayField : 'name',

  rootVisible : true,

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      root : {
        id : QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_ID,  // ROOT folder ID
        folderId : QDesk.util.constant.Const.MESSAGE_CHANNEL_PROFILES_ROOT_FOLDER_ID,
        name : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.rootNodeName' ),
        expanded : true,

        /* MessageChannelProfilesTreeNode */
        nodeType : 'folder'
      },

      store : Ext.create( 'QDesk.store.message.MessageChannelProfilesTreeStore' ),

      dockedItems : [
        {
          xtype : 'toolbar',
          itemId : 'toolBar',
          dock : 'top',
          items : [
            {
              xtype : 'button',
              itemId : 'refreshButton',
              scale : 'small',
              iconCls : 'refresh',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.tooltipRefreshButton' ),
              disabled : true
            },
            {
              xtype : 'button',
              itemId : 'collapseButton',
              scale : 'small',
              iconCls : 'collapse',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.tooltipCollapseButton' ),
              disabled : true
            },
            '-',
            {
              xtype : 'button',
              itemId : 'addFolderOrProfileButton',
              scale : 'small',
              iconCls : 'addFolderOrProfile',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.tooltipAddFolderOrProfileButton' ),
              disabled : true,
              menu : {
                xtype : 'menu',
                ignoreParentClicks : true,
                items : [
                  {
                    itemId : 'addFolderButton',
                    text : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.addFolderButton' ),
                    iconCls : 'addFolder'
                  },
                  {
                    itemId : 'addProfileButton',
                    text : QDesk.Msg.get( 'QDesk.view.main.message.channel.MessageChannelProfilesTreePanel.addProfileButton' ),
                    iconCls : 'addProfile',
                    menu : [
                      {
                        itemId : 'addEmailProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.EMAIL ),
                        iconCls : QDesk.enums.MessageChannel.EMAIL
                      },
                      '-',
                      {
                        itemId : 'addImAimProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.IM_AIM ),
                        iconCls : QDesk.enums.MessageChannel.IM_AIM
                      },
                      {
                        itemId : 'addImGTalkProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.IM_GTALK ),
                        iconCls : QDesk.enums.MessageChannel.IM_GTALK
                      },
                      {
                        itemId : 'addImIcqProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.IM_ICQ ),
                        iconCls : QDesk.enums.MessageChannel.IM_ICQ
                      },
                      {
                        itemId : 'addImSkypeProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.IM_SKYPE ),
                        iconCls : QDesk.enums.MessageChannel.IM_SKYPE
                      },
                      {
                        itemId : 'addImXmppProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.IM_XMPP ),
                        iconCls : QDesk.enums.MessageChannel.IM_XMPP
                      },
                      {
                        itemId : 'addImYahooProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.IM_YAHOO ),
                        iconCls : QDesk.enums.MessageChannel.IM_YAHOO
                      },
                      '-',
                      {
                        itemId : 'addWebServiceProfileButton',
                        text : QDesk.util.EnumUtils.getMessageChannelName( QDesk.enums.MessageChannel.WEB_SERVICE ),
                        iconCls : QDesk.enums.MessageChannel.WEB_SERVICE
                      }
                    ]
                  }
                ]
              }
            },
            {
              xtype : 'button',
              itemId : 'deleteFolderOrProfileButton',
              scale : 'small',
              iconCls : 'deleteFolderOrProfile',
              // no tooltip, it is set dynamically based on the selection (folder / profile)
              disabled : true
            }
          ]
        }
      ]
    } );

    me.callParent( [config] );
  }
} );
