/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.history.SchedulerExecHistoryTab', {
  extend : 'QDesk.view.base.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Config',
    'QDesk.Msg',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid'
  ],

  alias : 'widget.QuartzSchedulerExecHistoryTab',

  iconCls : 'quartzSchedulerExecHistorySubTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( config.qdesk.jvmAgentAvailable == null )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentAvailable' );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.SchedulerExecutionHistory )
    {
      if ( config.qdesk.jvmAgentAvailable )
      {
        Ext.apply( config, {
          layout : 'fit',
          items : [
            {
              xtype : 'QuartzExecHistoryGrid',
              itemId : 'execHistoryGrid',
              qdesk : {
                gridId : 'quartz.schedulerExecHistoryGrid',
                maxFetchSize : QDesk.Config.get( 'quartz.schedulerExecHistoryGrid.maxFetchSize' )
              }
            }
          ]
        } );
      }
      else
      {
        Ext.apply( config, {
          items : [
            {
              xtype : 'FeatureAccessRestrictedPanel',
              qdesk : {
                msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.jvmAgentNotAvailableMsg' )
              }
            }
          ]
        } );
      }
    }
    else
    {
      Ext.apply( config, {
        items : [
          {
            xtype : 'FeatureAccessRestrictedPanel',
            qdesk : {
              msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.notLicensedMsg',
                  QDesk.util.license.LicensedProduct.productName )
            }
          }
        ]
      } );
    }

    Ext.apply( config, {
      title : me._getTitle()
    } );

    me.callParent( [config] );
  },


  _getTitle : function ()
  {
    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var title = QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.SchedulerExecHistoryTab.title' );
    if ( !licensedFeatures.SchedulerExecutionHistory )
    {
      title = '<span class="notLicensed">' + title + '</span>'
    }

    return title;
  }
} );