/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.settings.AppSettingsTreePanel', {
  extend : 'Ext.tree.Panel',

  requires : [
    'QDesk.Msg',
    'QDesk.model.AppSettingsTreeNode',
    'QDesk.util.EnumUtils',
    'QDesk.util.constant.Const'
  ],

  alias : 'widget.AppSettingsTreePanel',

  cls : 'appSettingsTreePanel',

  autoScroll : true,

  header: false,

  useArrows : true,

  rootVisible : false,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    var store = Ext.create( 'Ext.data.TreeStore', {

      model : 'QDesk.model.AppSettingsTreeNode',

      root : {
        id : QDesk.util.constant.Const.APP_SETTINGS_ROOT_ID,  // ROOT folder ID
        name : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsTreePanel.rootNodeName' ),
        expanded : true,

        children : [
          /* Colors */
          {
            id : QDesk.util.constant.Const.APP_SETTINGS_COLORS_NODE_ID,
            name : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsTreePanel.colors' ),
            children : [
              {
                id : QDesk.util.constant.Const.APP_SETTINGS_COLORS_EXEC_HISTORY_NODE_ID,
                name : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsTreePanel.colors.execHistory' ),
                leaf : true
              }
            ]
          },

          /* Confirmations */
          {
            id : QDesk.util.constant.Const.APP_SETTINGS_CONFIRMATIONS_NODE_ID,
            name : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsTreePanel.confirmations' ),
            children : [
              {
                id : QDesk.util.constant.Const.APP_SETTINGS_CONFIRMATIONS_QUARTZ_NODE_ID,
                name : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsTreePanel.confirmations.quartz' ),
                leaf : true
              }
            ]
          },

          /* Notifications */
          {
            id : QDesk.util.constant.Const.APP_SETTINGS_NOTIFICATIONS_NODE_ID,
            name : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsTreePanel.notifications' ),
            leaf : true
          },

          /* System */
          {
            id : QDesk.util.constant.Const.APP_SETTINGS_SYSTEM_NODE_ID,
            name : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsTreePanel.system' ),
            leaf : true
          }
        ]
      }
    } );

    Ext.apply( config, {
      store : store,

      columns : [
        {
          xtype : 'treecolumn',
          dataIndex : 'name',
          flex : 1,
          renderer : function ( value, metaData, record )
          {
            var invalidFieldCount = record.get( 'invalidFieldCount' );
            if ( invalidFieldCount > 0 )
            {
              return '<span class="invalid">' + value + '</span>'
            }
            else
            {
              return '<span>' + value + '</span>'
            }
          }
        }
      ]

//    In the future we might want to provide config option live search capability in the settings tree.
//
//      dockedItems : [
//        {
//          xtype : 'toolbar',
//          itemId : 'toolBar',
//          dock : 'top',
//          items : [
//          ]
//        }
//      ]
    } );

    me.callParent( [config] );
  }
} );
