/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.settings.AppSettingsWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.Msg',
    'QDesk.view.main.settings.AppSettingsTreePanel',
    'QDesk.view.main.settings.colors.ColorsPanel',
    'QDesk.view.main.settings.colors.ColorsExecHistoryPanel',
    'QDesk.view.main.settings.confirm.ConfirmationsPanel',
    'QDesk.view.main.settings.confirm.QuartzConfirmationsPanel',
    'QDesk.view.main.settings.notif.NotificationsPanel',
    'QDesk.view.main.settings.system.SystemPanel'
  ],

  alias : 'widget.AppSettingsWindow',

  cls : 'appSettingsWindow',
  iconCls : 'appSettingsWindowIcon',

  modal : true,
  resizable : true,

  width : 700,
  minWidth : 600,

  height : 500,
  minHeight : 400,

  layout : {
    type : 'border'
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsWindow.windowTitle' ),

      items : [
        {
          xtype : 'AppSettingsTreePanel',
          itemId : 'appSettingsTreePanel',
          region : 'west',
          split : true,
          header : false,
          collapsible : true,
          collapseMode : 'mini',
          width : 250
        },
        {
          xtype : 'panel',
          itemId : 'cardPanel',
          region : 'center',
          layout : 'card',
          items : [
            {
              xtype : 'panel',
              itemId : 'infoPanel',
              border : false,
              cls : 'infoPanel',
              bodyPadding : '10px',
              html : QDesk.Msg.get( 'QDesk.view.main.settings.AppSettingsWindow.infoText' )
            },

            /* Confirmations */
            {
              xtype : 'ConfirmationsPanel',
              itemId : 'confirmationsPanel',
              qdesk : {
                nodeId : QDesk.util.constant.Const.APP_SETTINGS_CONFIRMATIONS_NODE_ID
              }
            },

            /* Quartz Confirmations */
            {
              xtype : 'QuartzConfirmationsPanel',
              itemId : 'quartzConfirmationsPanel',
              qdesk : {
                nodeId : QDesk.util.constant.Const.APP_SETTINGS_CONFIRMATIONS_QUARTZ_NODE_ID
              }
            },


            /* Colors */
            {
              xtype : 'ColorsPanel',
              itemId : 'colorsPanel',
              qdesk : {
                nodeId : QDesk.util.constant.Const.APP_SETTINGS_COLORS_NODE_ID
              }
            },
            {
              xtype : 'ColorsExecHistoryPanel',
              itemId : 'colorsExecHistoryPanel',
              qdesk : {
                nodeId : QDesk.util.constant.Const.APP_SETTINGS_COLORS_EXEC_HISTORY_NODE_ID
              }
            },


            /* Notifications */
            {
              xtype : 'NotificationsPanel',
              itemId : 'notificationsPanel',
              qdesk : {
                nodeId : QDesk.util.constant.Const.APP_SETTINGS_NOTIFICATIONS_NODE_ID
              }
            },


            /* System */
            {
              xtype : 'SystemPanel',
              itemId : 'systemPanel',
              qdesk : {
                nodeId : QDesk.util.constant.Const.APP_SETTINGS_SYSTEM_NODE_ID
              }
            }
          ]
        }
      ],

      dockedItems : [
        {
          xtype : 'toolbar',
          dock : 'bottom',
          ui : 'footer',
          defaults : {
            minWidth : 75
          },
          items : [
            {
              xtype : 'component',
              flex : 1
            },
            {
              xtype : 'button',
              itemId : 'okButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelOk' )
            },
            {
              xtype : 'button',
              itemId : 'cancelButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
            },
            {
              xtype : 'button',
              itemId : 'applyButton',
              text : QDesk.Msg.get( 'QDesk.buttonLabelApply' ),
              disabled : true  // disabled by default and enabled upon app settings changes
            }
          ]
        }
      ]
    } );

    me.callParent( [config] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  }
} )
;
