/*
 * Copyright (c) 2011-2013 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',
    'QDesk.view.base.BaseFormPanel'
  ],

  alias : 'widget.EditSchedulerConnectionForm',

  fieldDefaults : {
    labelWidth : 60
  },

  qdesk : {
    /*
     * Mode of the form (see EditConnectionFolderFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    // initially empty store used by the schedulerObjectName combo
    var schedulerObjectNameStore = Ext.create( 'Ext.data.Store', {
      fields : ['objectName'],
      data : []
    } );

    Ext.apply( config, {
      items : [
        {
          xtype : 'hiddenfield',
          name : 'id'
        },
        {
          xtype : 'textfield',
          itemId : 'name',
          name : 'name',
          fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldLabelName' ),
          allowBlank : false
        },
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldsetLabelServer' ),
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'host',
              name : 'host',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldLabelHost' ),
              allowBlank : false
            },
            {
              xtype : 'numberfield',
              itemId : 'port',
              name : 'port',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldLabelPort' ),
              allowBlank : false,
              minValue : 1,
              maxValue : 65535
            },
            {
              xtype : 'fieldset',
              collapsible : true,
              collapsed : true,
              title : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldsetLabelAuthentication' ),
              layout : 'anchor',
              defaults : {
                anchor : '100%'
              },
              items : [
                {
                  xtype : 'textfield',
                  itemId : 'username',
                  name : 'username',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldLabelUsername' ),
                  allowBlank : true
                },
                {
                  xtype : 'textfield',
                  itemId : 'password',
                  name : 'password',
                  fieldLabel : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldLabelPassword' ),
                  inputType : 'password',
                  allowBlank : true
                }
              ]
            }
          ]
        },
        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.fieldsetLabelSchedulerObjectName' ),
          layout : 'anchor',
          defaults : {
            anchor : '100%'
          },
          items : [
            {
              xtype : 'panel',
              border : false,
              layout : {
                type : 'hbox'
              },
              style : {
                marginBottom : '5px'
              },
              items : [
                {
                  xtype : 'combo',
                  itemId : 'schedulerObjectName',
                  name : 'schedulerObjectName',
                  allowBlank : false,
                  flex : 1,
                  queryMode : 'local',
                  store : schedulerObjectNameStore,
                  valueField : 'objectName',
                  displayField : 'objectName'
                },
                {
                  xtype : 'button',
                  itemId : 'discoverButton',
                  margin : '0 0 0 5',
                  text : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.buttonLabelDiscover' ),
                  disabled : true
                }
              ]
            }
          ]
        },

        {
          xtype : 'panel',
          border : false,
          layout : {
            type : 'hbox',
            pack : 'end'
          },
          items : [
            {
              xtype : 'button',
              itemId : 'testButton',
              text : QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionForm.buttonLabelTest' ),
              formBind : true
            }
          ]
        }
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true
        }
      ]
    } );

    me.callParent( [config] );

    me.down( '#host' ).on( 'change', me._maybeEnableButtons, me );
    me.down( '#port' ).on( 'change', me._maybeEnableButtons, me );
    me.down( '#schedulerObjectName' ).on( 'change', me._maybeEnableButtons, me );
  },


  _getActionButtonText : function ( mode )
  {
    var me = this;

    switch ( mode )
    {
      case QDesk.view.navigation.connections.EditSchedulerConnectionFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.navigation.connections.EditSchedulerConnectionFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }
  },


  _maybeEnableButtons : function ()
  {
    var me = this;

    var discoverButton = me.down( '#discoverButton' );
    var testButton = me.down( '#testButton' );

    var hostValue = me.down( '#host' ).getValue();
    var portValue = me.down( '#port' ).getValue();
    var schedulerObjectNameValue = me.down( '#schedulerObjectName' ).getValue();

    // disabled if either of the host, or port field is empty
    discoverButton.setDisabled( hostValue == '' || portValue == null );

    // disabled if either of the host, port, or object name field is empty
    testButton.setDisabled( hostValue == '' || portValue == null || schedulerObjectNameValue == null );
  }
} );
